
let socket;
let pc;
var remoteVideo;
var remoteTrack;

function connect() {
    //socket = new WebSocket("ws://192.168.3.177:9092");
    socket = new WebSocket("ws://" + location.hostname + ":9092");

    socket.onopen = function(e) {
        console.log("[open] Connection established");
        console.log("Sending to server");

        createPeerConnection();
        sendOffer();
    };

    socket.onmessage = function(event) {
        let msg = event.data;
        console.log(`[message] Data received from server: ${event.data}`);
        if (msg.startsWith('ANSWER')) {
            let answer = msg.substring(7)
            pc.setRemoteDescription(new RTCSessionDescription({
                type: 'answer',
                sdp: answer
            }));
        } else if (msg.startsWith('OFFER')) {
            let offer = msg.substring(6)
            pc.setRemoteDescription(new RTCSessionDescription({
                type: 'offer',
                sdp: offer
            }));
            sendAnswer();
        } else if (msg.startsWith('ICE')) {
            let ice = msg.substring(4)
            let parts = ice.split('$')
            console.log('ICE candidate received +' +ice);
            pc.addIceCandidate(new RTCIceCandidate({
                type: 'candidate',
                sdpMid: parts[0],
                sdpMLineIndex: parts[1],
                candidate: parts[2],
            }));
        } else if (msg.startsWith('DESQ')) {
            let desq = msg.substring(5)
            let scale = parseFloat(desq)
            remoteVideo.style.transform = 'scale(1, ' + scale + ')'
        }
    };

    socket.onclose = function(event) {
        if (event.wasClean) {
            console.log(`[close] Connection closed cleanly, code=${event.code} reason=${event.reason}`);
        } else {
            console.log('[close] Connection died');
        }
    };

    socket.onerror = function(error) {
        console.log(`[error] ${error.message}`);
    };
}

let black = ({ width = 1920, height = 1080 } = {}) => {
    let canvas = Object.assign(document.createElement("canvas"), { width, height });
    canvas.getContext('2d').fillRect(0, 0, width, height);
    let stream = canvas.captureStream(0);
    return Object.assign(stream.getVideoTracks()[0], { enabled: false });
}

let createPeerConnection = () => {
  try {
    pc = new RTCPeerConnection();
    pc.onicecandidate = onIceCandidate;
    pc.ontrack = onTrack;
    pc.addTrack(black({width:320, height:240}));
    //pc.addTrack(localStream.getTracks()[0]);
    console.log('PeerConnection created');
  } catch (error) {
    console.error('PeerConnection failed: ', error);
  }
};

let sendOffer = () => {
  console.log('Send offer');
  pc.createOffer({
    offerToReceiveVideo: true,
    offerToReceiveAudio: false,
  }).then(
    setAndSendLocalDescription('OFFER'),
    (error) => { console.error('Send offer failed: ', error); }
  );
};

let sendAnswer = () => {
    console.log('Send answer');
    pc.createAnswer({
        offerToReceiveVideo: true,
        offerToReceiveAudio: false,
    }).then(
      setAndSendLocalDescription('ANSWER'),
      (error) => { console.error('Send answer failed: ', error); }
    );
};

function setAndSendLocalDescription(type) {
    return (sessionDescription) => {
        pc.setLocalDescription(sessionDescription);
        console.log('Local description set' + sessionDescription.sdp);
        socket.send(type + ' ' + sessionDescription.sdp);
    };
}

let onIceCandidate = (event) => {
    if (event.candidate) {
        let candidate = event.candidate
        socket.send('ICE ' + candidate.sdpMid + '$' + candidate.sdpMLineIndex + '$' + candidate.candidate);
    }
};

let onTrack = (event) => {
  console.log('Add track');
  //remoteVideo.srcObject = event.streams[0];
  remoteVideo.onloadedmetadata = () => {

    let dimensions = {
        width: remoteVideo.videoWidth,
        height: remoteVideo.videoHeight
    };

    let settings = remoteTrack.getSettings();
    let constraints = remoteTrack.getConstraints();
    console.log(settings);
    console.log(constraints);
    console.log(dimensions);

};

  remoteTrack = event.track;
  remoteVideo.srcObject = new MediaStream([remoteTrack]);
  remoteVideo.muted = true;
  remoteVideo.play();
};

function pageReady() {
    remoteVideo = document.getElementById('remote-video');
    connect();
    console.log('now connect');
}
